from typing import List, Optional, Tuple

import numpy as np
import pandas as pd
from pydantic import BaseModel, Field, ValidationError

from red_wine_mm.config.core import config


def validate_inputs(*, input_data: pd.DataFrame) -> Tuple[pd.DataFrame, Optional[dict]]:
    """Check model inputs for unprocessable values."""

    validated_data = input_data[config.m_config.features].copy()
    errors = None

    try:
        # replace numpy nans so that pydantic can validate
        MultipleTitanicDataInputs(
            inputs=validated_data.replace({np.nan: None}).to_dict(orient="records")
        )
    except ValidationError as error:
        errors = error.json()

    return validated_data, errors


class WineDataInputSchema(BaseModel):
    fixed_acidity: Optional[float] = Field(alias="fixed acidity")
    volatile_acidity: Optional[float] = Field(alias="volatile acidity")
    chlorides: Optional[float]
    free_sulfur_dioxide: Optional[float] = Field(alias="free sulfur dioxide")
    total_sulfur_dioxide: Optional[float] = Field(alias="total sulfur dioxide")
    sulphates: Optional[float]
    alcohol: Optional[float]


class MultipleTitanicDataInputs(BaseModel):
    inputs: List[WineDataInputSchema]



from config.core import config
from pipeline import red_wine_pipe
from processing.data_manager import load_dataset, save_pipeline
from sklearn.model_selection import train_test_split


def run_training() -> None:
    """Train the model."""

    # read training data
    data = load_dataset(file_name=config.app_config.raw_data_file)

    # divide train and test
    X_train, X_test, y_train, y_test = train_test_split(
        data[config.m_config.features],  # predictors
        data[config.m_config.target],
        test_size=config.m_config.test_size,
        stratify=data[config.m_config.target],
        random_state=config.m_config.random_state,
    )

    # fit model
    red_wine_pipe.fit(X_train, y_train)

    # persist trained model
    save_pipeline(pipeline_to_persist=red_wine_pipe)


if __name__ == "__main__":
    run_training()



import typing as t

import pandas as pd

from red_wine_mm import __version__ as _version
from red_wine_mm.config.core import config
from red_wine_mm.processing.data_manager import load_pipeline
from red_wine_mm.processing.validation import validate_inputs

pipeline_file_name = f"{config.app_config.pipeline_save_file}{_version}.pkl"
red_wine_pipe = load_pipeline(file_name=pipeline_file_name)


def make_prediction(
    *,
    input_data: t.Union[pd.DataFrame, dict],
) -> dict:
    """Make a prediction using a saved model pipeline."""

    data = pd.DataFrame(input_data)
    validated_data, errors = validate_inputs(input_data=data)
    results = {"predictions": None, "version": _version, "errors": errors}

    if not errors:
        predictions = red_wine_pipe.predict(X=validated_data[config.m_config.features])
        results = {
            "predictions": predictions,
            "version": _version,
            "errors": errors,
        }

    return results



"""
Note: These tests will fail if you have not first trained the model.
"""

import numpy as np
from sklearn.metrics import accuracy_score

from red_wine_mm.predict import make_prediction


def test_make_prediction(sample_input_data):
    x_test, y_test = sample_input_data
    # Given
    expected_no_predictions = 159

    # When
    result = make_prediction(input_data=x_test)

    # Then
    predictions = result.get("predictions")
    assert isinstance(predictions, np.ndarray)
    assert isinstance(predictions[0], np.int64)
    assert result.get("errors") is None
    assert len(predictions) == expected_no_predictions
    _predictions = list(predictions)
    accuracy = accuracy_score(_predictions, y_test)
    print(f"accuracy score: {accuracy}")
    assert accuracy > 0.90



from sklearn.ensemble import RandomForestClassifier
from sklearn.model_selection import GridSearchCV
from sklearn.pipeline import Pipeline
from sklearn.preprocessing import StandardScaler

from red_wine_mm.config.core import config

red_wine_pipe = Pipeline(
    [
        ("scaler", StandardScaler()),
        (
            "random_forest_classifier",
            GridSearchCV(
                RandomForestClassifier(random_state=config.m_config.random_state),
                param_grid={"n_estimators": [50, 100, 200]},
                cv=5,
            ),
        ),
    ]
)
